/*
***************************************************************************
**                                                                       **
**	Restart manager  -  Extension for Pale Moon                          **
**                                                                       **
**	Customization mode code                                              **
**                                                                       **
**	Copyright(C) 2018 - 2020 by George Malamas                           **
**                                                                       **
**	Free software for all people                                         **
**                                                                       **
**	Last edit: June 20, 2020                                             **
**                                                                       **
**	Release number: 6                                                    **
**                                                                       **
***************************************************************************
*/
/*
Since we set dynamically on our toolbar items some of their attributes, we have to do it also during
 customization. Also, in appVer < 4, we set always default class on our restart button.
*/
// "use strict";
const {classes: Cc, interfaces: Ci} = Components;
var winMed = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
var rmToolbarBtnPos = undefined; // Restart button.
var cpnItemPosition = undefined; // Current profile name indicator.
var firstBtnSetupInPalette = false;
var buttonHostToolbar = undefined;
var buttonHostToolbarMode = undefined;
var verticalTlbrCustMode = false;

function customizationModeInit() {
	var mainBrowzerWin = null;
	if(window.opener && window.opener.document.documentElement.getAttribute("windowtype") == "navigator:browser") {
		mainBrowzerWin = window.opener;
	} else {
		mainBrowzerWin = winMed.getMostRecentWindow("navigator:browser");
	};
	var appVer = mainBrowzerWin._RestMan.appInfos.appVer;
	var rmToolbarBtn = null; // Restart button.
	var cpnToolbarItem = null; // Current profile name indicator.
	var boldWidth = 0;
	var restoreToolbars = false;

	function shared(item) {
		if(item.hasAttribute("icon")) {
			item.removeAttribute("icon");
		};
		if(item.childNodes[1].hasAttribute("mozNoRemote")) {
			item.childNodes[1].removeAttribute("mozNoRemote");
		};
		mainBrowzerWin.clearInterval(item.childNodes[1].getAttribute("updaterId"));
		item.childNodes[1].removeAttribute("updaterId");
		var toolboxPalette = mainBrowzerWin.document.getElementById("navigator-toolbox").palette;
		for(var n = toolboxPalette.childNodes.length - 1; n >= 0; n--) { // Reverse searching because
		// the just moved in palette item most likely will go at the end of the list.
			if(toolboxPalette.childNodes[n].id == "restman-profileName") {
				if(toolboxPalette.childNodes[n].hasAttribute("icon")) {
					toolboxPalette.childNodes[n].removeAttribute("icon");
				};
				if(toolboxPalette.childNodes[n].childNodes[1].hasAttribute("mozNoRemote")) {
					toolboxPalette.childNodes[n].childNodes[1].removeAttribute("mozNoRemote");
				};
				toolboxPalette.childNodes[n].childNodes[1].removeAttribute("updaterId");
				break;
			};
		};
	};

	function setItemAttr() {window.setTimeout(function() {
		if((rmToolbarBtn = mainBrowzerWin.document.getElementById("restman-button")) && // Button is on a toolbar.
		// Not mistyped "=" above! It's an assignment in conditional expression, used also in other 3 cases below in this document.
		   ((rmToolbarBtnPos == "palette") || verticalTlbrCustMode || (buttonHostToolbar !==
		   rmToolbarBtn.parentNode.parentNode.parentNode.id) || (buttonHostToolbarMode !==
		   rmToolbarBtn.parentNode.parentNode.parentNode.getAttribute("mode")))) {
			rmToolbarBtnPos = "toolbar";
			buttonHostToolbar = rmToolbarBtn.parentNode.parentNode.parentNode.id;
			buttonHostToolbarMode = rmToolbarBtn.parentNode.parentNode.parentNode.getAttribute("mode");
			mainBrowzerWin._RestMan.onLoadResources(); // Some of our prefs, used here, must be refreshed.
			rmToolbarBtn.label = mainBrowzerWin._RestMan.locals.restartLabel;
			rmToolbarBtn.setAttribute("tooltiptext", mainBrowzerWin._RestMan.locals.restartLabel);
			rmToolbarBtn.setAttribute("iconStyle", mainBrowzerWin._RestMan.locals.iconStyle);
			if(mainBrowzerWin._RestMan.locals.invCachHghlght) {
				rmToolbarBtn.setAttribute("highlightBtn", true);
			};
			if(appVer >= 4) {
				if((mainBrowzerWin._RestMan.locals.rmButtonStyle == 1) && (rmToolbarBtn.getAttribute("class") !== "restman-toolbarbutton")) {
					rmToolbarBtn.setAttribute("class", "restman-toolbarbutton");
				} else if((mainBrowzerWin._RestMan.locals.rmButtonStyle == 0) && (rmToolbarBtn.getAttribute("class") !== "toolbarbutton-1")) {
					rmToolbarBtn.setAttribute("class", "toolbarbutton-1");
				};
				if(mainBrowzerWin._RestMan.locals.rmSmallButton) {
					rmToolbarBtn.setAttribute("smallButton", true);
				};
				if(appVer >= 22) {
					rmToolbarBtn.setAttribute("appVer22plus", "");
				};
				if(appVer >= 14) {
					rmToolbarBtn.setAttribute("appVer14plus", "");
				} else {
					rmToolbarBtn.setAttribute("appVer4-13", "");
				};
				// We set the width of our button's label to be always as when the label is bold, to avoid the button to be
				// expanded/reduced when the label becomes bold/normal. This way, the extra space of label when it is normal
				// "absorbs" the extra width when it becomes bold.
				var btnLabel = rmToolbarBtn.ownerDocument.getAnonymousNodes(rmToolbarBtn)[1];
				if((buttonHostToolbarMode == "full") || (buttonHostToolbarMode == "text")) {
					if(!mainBrowzerWin._RestMan.locals.invCachHghlght) {
						rmToolbarBtn.setAttribute("highlightBtn", true); // Makes the label "bold"
					};
					if(btnLabel.getAttribute("class") == "toolbarbutton-text") { // Just for peace of mind..
						if(btnLabel.hasAttribute("style")) {
							btnLabel.removeAttribute("style"); // This is necessary on "customizationchange", if our button is in "Vertical Toolbar".
						}; // Otherwise, our button's label will take bigger width, if some other button with longer label is removed.
						// boldWidth = Math.round(parseInt(window.getComputedStyle(btnLabel).getPropertyValue("width")));
						boldWidth = btnLabel.clientWidth + 1; // Simpler than above & accurate, in all supported appVer.
					};
					if(!mainBrowzerWin._RestMan.locals.invCachHghlght) {
						rmToolbarBtn.removeAttribute("highlightBtn");
					};
				} else {
					boldWidth = 0;
				};
				// Compatibility mainly with "Vertical Toolbar".
				if((mainBrowzerWin.getComputedStyle(rmToolbarBtn, null).MozBoxOrient == "horizontal") || // On customizing via browser's customization mode or "Vertical Toolbar".
				   (rmToolbarBtn.parentNode.parentNode.parentNode.getAttribute("mode") == "text") || // On customizing via browser's customization mode.
				   (rmToolbarBtn.parentNode.parentNode.getAttribute("mode") == "text")) { // On customizing via "Vertical Toolbar".
					rmToolbarBtn.parentNode.removeAttribute("align");
				} else {
					rmToolbarBtn.parentNode.setAttribute("align", "center");
				};
			} else if(appVer < 4) {
				rmToolbarBtn.setAttribute("class", "toolbarbutton-1");
			};
			if((appVer < 4) && mainBrowzerWin.document.getElementById("wrapper-restman-toolbaritem")) { // Our "toolbarpaletteitem" just placed on a toolbar.
				// AppVer < 4 don't have by default attribute "iconsize" in all toolbars, or it's not set (in additional toolbars),
				// until the user check for first time "Use Small Icons" in customization window.
				var itemInToolbar = mainBrowzerWin.document.getElementById("wrapper-restman-toolbaritem");
				if(itemInToolbar.parentNode.getAttribute("iconsize") == "") {
					itemInToolbar.setAttribute("hidden", true);
					window.updateIconSize(false); // customizeToolbar.js
					itemInToolbar.removeAttribute("hidden");
				};
			};
		} else if((rmToolbarBtn = window.document.getElementById("restman-button")) && // Button is in customization palette.
			((rmToolbarBtnPos == "toolbar") || !firstBtnSetupInPalette || restoreToolbars)) {
			restoreToolbars = false;
			mainBrowzerWin._RestMan.onLoadResources(); // Some of our prefs, used here, must be refreshed.
			var rmToolbarpaletteitem = window.document.getElementById("wrapper-restman-toolbaritem");
			if(rmToolbarBtn.hasAttribute("highlightBtn")) {
				rmToolbarBtn.removeAttribute("highlightBtn");
			};
			if((appVer >= 4) && rmToolbarpaletteitem) {
				if(rmToolbarBtn.getAttribute("class") !=="restman-toolbarbutton") {
					rmToolbarBtn.setAttribute("class", "restman-toolbarbutton");
				};
				rmToolbarpaletteitem.setAttribute("title", mainBrowzerWin._RestMan.locals.restartLabel);
				rmToolbarpaletteitem.setAttribute("tooltiptext", mainBrowzerWin._RestMan.locals.restartTitle);
			} else if(rmToolbarpaletteitem) {
				rmToolbarBtn.setAttribute("class", "toolbarbutton-1");
				rmToolbarBtn.label = mainBrowzerWin._RestMan.locals.restartLabel;
				rmToolbarBtn.setAttribute("tooltiptext", mainBrowzerWin._RestMan.locals.restartLabel);
			};
			var toolboxPalette = mainBrowzerWin.document.getElementById("navigator-toolbox").palette;
			for(var n = toolboxPalette.childNodes.length - 1; ((n >= 0) && (rmToolbarBtnPos == "toolbar")); n--) { // Reverse searching.
				if((toolboxPalette.childNodes[n].id == "restman-toolbaritem") &&
				  (toolboxPalette.childNodes[n].childNodes[1].hasAttribute("highlightBtn"))) {
					toolboxPalette.childNodes[n].childNodes[1].removeAttribute("highlightBtn");
					break;
				};
			};
			rmToolbarBtnPos = "palette";
			firstBtnSetupInPalette = true;
		};
		if((cpnToolbarItem = mainBrowzerWin.document.getElementById("restman-profileName")) && // Item is on a toolbar.
		   (cpnItemPosition == "palette")) {
			cpnItemPosition = "toolbar";
			var getInfo = undefined;
			try {
				getInfo = new CustomEvent("getInfo"); // "CustomEvent()" constructor, appVer ~11+.
			} catch(ex) {
				getInfo = window.document.createEvent("Event");
				getInfo.initEvent("getInfo", true, true);
			};
			cpnToolbarItem.childNodes[1].value = mainBrowzerWin._RestMan.restmanProfMan(getInfo)[0];
			var currentPrefs = {
				icon: mainBrowzerWin._RestMan.rmPrefBranch.getBoolPref("profileName_icon"),
				bold: mainBrowzerWin._RestMan.rmPrefBranch.getBoolPref("profileName_bold"),
				ital: mainBrowzerWin._RestMan.rmPrefBranch.getBoolPref("profileName_italic"),
				color: mainBrowzerWin._RestMan.rmPrefBranch.getCharPref("profileName_color")
			};
			if(!currentPrefs.icon) {
				cpnToolbarItem.setAttribute("icon", false);
			};
			if(currentPrefs.bold) {
				cpnToolbarItem.childNodes[1].style.fontWeight = "bold";
			};
			if(currentPrefs.ital) {
				cpnToolbarItem.childNodes[1].style.fontStyle = "italic";
			};
			if(currentPrefs.color) {
				cpnToolbarItem.childNodes[1].style.color = currentPrefs.color;
			};
			if(mainBrowzerWin._RestMan.restmanSrvc.getSet("startedWithNoRemote", "getV")) { // get value.
				cpnToolbarItem.childNodes[1].setAttribute("mozNoRemote", true);
			};
		} else if((cpnToolbarItem = window.document.getElementById("restman-profileName")) && // Item is in customization palette.
		   (cpnItemPosition == "toolbar")) {
			cpnItemPosition = "palette";
			return shared(cpnToolbarItem);
		};
	}, 0);};

	function setItemAttr2() {window.setTimeout(function() { // AppVer 1.5 - 3.6.
		if((rmToolbarBtn = window.document.getElementById("restman-button")) && // Button is in customization palette.
		   ((rmToolbarBtnPos == "toolbar") || restoreToolbars)) {
			rmToolbarBtnPos = "palette";
			restoreToolbars = false;
			rmToolbarBtn.label = mainBrowzerWin._RestMan.locals.restartLabel;
			rmToolbarBtn.setAttribute("tooltiptext", mainBrowzerWin._RestMan.locals.restartLabel);
			rmToolbarBtn.setAttribute("class", "toolbarbutton-1");
			if(rmToolbarBtn.hasAttribute("highlightBtn")) {
				rmToolbarBtn.removeAttribute("highlightBtn");
				var toolboxPalette = mainBrowzerWin.document.getElementById("navigator-toolbox").palette;
				for(var n = toolboxPalette.childNodes.length - 1; n >= 0; n--) { // Reverse searching.
				if((toolboxPalette.childNodes[n].id == "restman-toolbaritem") &&
				  (toolboxPalette.childNodes[n].childNodes[1].hasAttribute("highlightBtn"))) {
					toolboxPalette.childNodes[n].childNodes[1].removeAttribute("highlightBtn");
						break;
					};
				};
			};
		};
		if((cpnToolbarItem = window.document.getElementById("restman-profileName")) && // Item is in customization palette.
		   (cpnItemPosition == "toolbar")) {
			cpnItemPosition = "palette";
			return shared(cpnToolbarItem);
		};
	}, 0);};

	function restoreToolbarsDefaultNotify() {
		restoreToolbars = true;
		if(appVer < 4) { // No customization change events ("drop/dragdrop") on "Restore Default Set".
			return setItemAttr2();
		};
	};

	var btnInPalette = window.document.getElementById("restman-button");
	var btnOnToolbar = mainBrowzerWin.document.getElementById("restman-button");
	var cpnInPalette =window.document.getElementById("restman-profileName");
	var cpnOnToolbar = mainBrowzerWin.document.getElementById("restman-profileName");
	if(btnInPalette || btnOnToolbar || cpnInPalette || cpnOnToolbar) {
		if(window.document.getElementById("CustomizeToolbarWindow")) {
			if(btnInPalette) {
				rmToolbarBtnPos = "palette";
			} else if(btnOnToolbar) {
				rmToolbarBtnPos = "toolbar";
			};
			if(appVer >= 3.5) {
				if(cpnInPalette) {
					cpnItemPosition = "palette";
				} else if(cpnOnToolbar) {
					cpnItemPosition = "toolbar";
				};
			};
			if(appVer >= 4) {
				mainBrowzerWin.addEventListener("customizationchange", setItemAttr, false); // AppVer >=4.
			} else if((appVer < 4) && (appVer >= 3.5)) { // AppVer 3.5 & 3.6.
				mainBrowzerWin.addEventListener("drop", setItemAttr, true);
				window.addEventListener("drop", setItemAttr2, true);
			} else { // AppVer 1.5 - 3.
				mainBrowzerWin.addEventListener("dragdrop", setItemAttr, true);
				window.addEventListener("dragdrop", setItemAttr2, true);
			};
			var restoreButton = window.document.getElementById("restoreDefault") || // AppVer 15+.
				window.document.getElementById("main-box").getElementsByAttribute("oncommand", "restoreDefaultSet();")[0];
			restoreButton.addEventListener("command", restoreToolbarsDefaultNotify, false);
			window.addEventListener("unload", function() {
				if(appVer >= 4) {
					mainBrowzerWin.removeEventListener("customizationchange", setItemAttr, false); // AppVer >=4.
					if((boldWidth > 0) && (rmToolbarBtn = mainBrowzerWin.document.getElementById("restman-button"))) {
						// Toolbar mode = "full" or "text". We set here the width, otherwise will not be kept.
						rmToolbarBtn.ownerDocument.getAnonymousNodes(rmToolbarBtn)[1].style.width = boldWidth + "px";
					};
				} else if((appVer < 4) && (appVer >= 3.5)) { // AppVer 3.5 & 3.6.
					mainBrowzerWin.removeEventListener("drop", setItemAttr, true);
					mainBrowzerWin._RestMan._RestManOnLoad(true);
				} else { // AppVer 1.5 - 3.
					mainBrowzerWin.removeEventListener("dragdrop", setItemAttr, true);
					mainBrowzerWin._RestMan._RestManOnLoad(true);
				};
				restoreButton.removeEventListener("command", restoreToolbarsDefaultNotify, false);
			}, false);
			return setItemAttr();
		} else if((appVer >= 24) && window.document.getElementById("verticaltoolbarPrefs")) { // Our customization overlay is
			// applied also to "Vertical Toolbar" extension's pref window.
			rmToolbarBtn = mainBrowzerWin.document.getElementById("restman-button");
			function onCommad() {
				if(rmToolbarBtn && (rmToolbarBtn.parentNode.parentNode.id == "vertical-toolbar")) {
					return setItemAttr();
				};
			};
			var buttonMode = window.document.getElementById("button_mode");
			if(rmToolbarBtn) {
				verticalTlbrCustMode = true;
				buttonMode.addEventListener("command", onCommad, false);
			};
			window.onunload = function() {
				if(verticalTlbrCustMode) {
					buttonMode.removeEventListener("command", onCommad, false);
				};
			};
		};
	};
};

