/*
***************************************************************************
**                                                                       **
**	Restart manager  -  Extension for Pale Moon                          **
**                                                                       **
**	Main code to manage restart or start requests for other profiles     **
**                                                                       **
**	Copyright(C) 2020 by George Malamas                                  **
**                                                                       **
**	Free software for all people                                         **
**                                                                       **
**	Last edit: July 6, 2020                                              **
**                                                                       **
**	Release number: 1                                                    **
**                                                                       **
***************************************************************************
*/
// First, we try to read the profiles.ini file and thereafter we compare the result with
// known by current app instance profile data (As offered by profile service). We do this
// because, if the profile data modified by another app instance or by profile manager or
// even by manually editing profiles.ini, the current app instance will not be aware of it.
// In case of data match after the comparison (most likely), we try to use the offered by
// profile service data, while in case of mismatch we try to use the gathered data from
// profiles.ini.
// Even in case of data match, we still try to use some profiles.ini data in some checks
// we perform, provided that we have found and read the profiles.ini file successfully.

var profileSrv = null;
if("@mozilla.org/toolkit/profile-service;1" in Components.classes) { // Profile service is accessible only by
// profile manager in some oldies..
	profileSrv = cc["@mozilla.org/toolkit/profile-service;1"].createInstance(ci.nsIToolkitProfileService);
};
var profilesIniData = new Array(); // Current profile data in "profiles.ini".
var profilesIniPath = undefined;

function profIniModWatcher() { // Watches periodically if profiles.ini has been modified and updates
// the name of the current profile in our toolbar item, if it is present on a toolbar.
	var cpn = window.document.getElementById("restman-profileName"); // Current profile name indicator toolbar item.
	if(!cpn || cpn.childNodes[1].hasAttribute("updaterId")) {
		return;
	};
	var profIniNsIFile = cc["@mozilla.org/file/local;1"].createInstance(ci.nsIFile);
	if(!("initWithPath" in profIniNsIFile)) { // oldies..
		profIniNsIFile = cc["@mozilla.org/file/local;1"].createInstance(ci.nsILocalFile);
	};
	if(profilesIniPath) {
		profIniNsIFile.initWithPath(profilesIniPath);
		var lastTimeStamp = profIniNsIFile.lastModifiedTime;
		var updaterId = window.setInterval(function() {
			if(profIniNsIFile.exists()) {
				profIniNsIFile.initWithPath(profilesIniPath);
				var currentTimeStamp = profIniNsIFile.lastModifiedTime;
				if(lastTimeStamp < currentTimeStamp) {
					cpn.childNodes[1].value = profileListCreate(false)[0]; // Update current profile name.
					lastTimeStamp = currentTimeStamp;
				};
			};
		}, 5000);
		cpn.childNodes[1].setAttribute("updaterId", updaterId);
	};
};

function profileListCreate(mItems) { // We create the two profile lists every time our context menu is opened
// and then we destroy them. This way the lists are not kept always in memory and they are always up to date. 
	var devBundle = _RestMan.strings.createBundle("chrome://pippki/locale/pippki.properties"); // "Path" string.
	var pathStr = "Path";
	try {
		pathStr = devBundle.GetStringFromName("devinfo_modpath"); // "Path" string.
	} catch(ex) {};
	var currentProfileName = "- - -";
	var profilesInUse = 0;
	function menuitemConstructor(mItemAttr) {
		var mItem = window.document.createElement("menuitem");
		mItem.setAttribute("class", "menuitem-iconic");
		mItem.setAttribute("label", mItemAttr.label);
		mItem.setAttribute("accesskey", mItemAttr.label.charAt(0));
		mItem.setAttribute("tooltiptext", mItemAttr.ttt);
		mItem.setAttribute("oncommand", "_RestMan.restmanProfMan(event);"); // event.button.....!!!!!!!
		if(mItemAttr.label2) {
			mItem.setAttribute("acceltext", mItemAttr.label2); // "acceltext" is used here as a
			// hackish way to put a second label at the end of the menu item.
		} else {
			mItem.setAttribute("acceltext", "\u0020"); // By putting "acceltext" in all menu items, makes
		}; // the container behave as if were xul columns, separating vertically the two labels of menu items.
		if(mItemAttr.disabled) {
			mItem.setAttribute("disabled", true); // "current", "in use" or "invalid" profiles.
		};
		return mItem;
	};
	var invalidPaths = new Array();
	function readProfilesIni() {
		var defProfRoot = (_RestMan.appInfos.targetOS == "Linux") ?
			_RestMan.dirSrvc.get("DefProfRt", Ci.nsIFile) :
			_RestMan.dirSrvc.get("DefProfRt", Ci.nsIFile).parent;
		var profIni = defProfRoot.clone();
		profIni.append("profiles.ini");
		if(!profIni.exists()) {
			_RestMan.logger("Restart manager: The \"profiles.ini\" file could not be found.");
			return false;
		};
		profilesIniPath = profIni.path;
		var iniParser = cc["@mozilla.org/xpcom/ini-parser-factory;1"].getService(ci.nsIINIParserFactory).createINIParser(profIni);
		var secttionsEnum = iniParser.getSections();
		while(secttionsEnum && secttionsEnum.hasMore()) {
			var section = secttionsEnum.getNext();
			if(!(/Profile\d+/).test(section)) {
				continue;
			};
			var sectionKeys = {};
			var keysEnum = iniParser.getKeys(section);
			while(keysEnum && keysEnum.hasMore()) {
				sectionKeys.iniOrder = window.parseInt(section.match(/\d+/)[0], 10);
				var key = keysEnum.getNext();
				if(["Name", "IsRelative", "Path", "Default"].indexOf(key) > -1) {
					sectionKeys[key] = iniParser.getString(section, key);
				};
			};
			if(("Path" in sectionKeys) && sectionKeys.Path) {
				if("IsRelative" in sectionKeys) {
					if(sectionKeys.IsRelative == "1") {
						if((/\\/).test(sectionKeys.Path)) {
							invalidPaths.push(sectionKeys.Path);
						} else {
							var profRootDir = defProfRoot.clone();
							var relativePathSegments = sectionKeys.Path.split("/");
							relativePathSegments.forEach(function(segment) { // Getting absolute path from relative path.
								try {
									profRootDir.append(segment);
								} catch(ex) {
									invalidPaths.push(profRootDir.path);
								};
							});
							sectionKeys.Path = profRootDir.path;
						};
					} else if(sectionKeys.IsRelative != "0") {
						invalidPaths.push(sectionKeys.Path);
					};
				} else {
					invalidPaths.push(sectionKeys.Path);
				};
			};
			delete sectionKeys.IsRelative;
			profilesIniData.push(sectionKeys);
		};
		profilesIniData.sort(function(a, b) { // The occurred list by iniParser is sorted in a weird
		// way. We sort it in ascending order by profile number, the same way profile manager does.
			return a.iniOrder - b.iniOrder;
		});
		var lastDefault = -1;
		profilesIniData.forEach(function(obj) { // If there are more than 1 profiles marked as
		// "Default", we keep the last, as profile manager also does.
			if("Default" in obj) {
				if(obj.Default == "1") {
					if((lastDefault >= 0) && (obj.iniOrder > lastDefault)) {
						delete profilesIniData[lastDefault].Default;
					};
					lastDefault = obj.iniOrder;
				} else {
					delete obj.Default; // Normally, only a "Default=1" key/value pair should exist in profiles.ini.
				};
			} else if(profilesIniData.length == 1) { // In some app versions, if there is only 1 profile, it is not
				// marked as "Default" in profiles.ini ("Default=1" key/value pair is missing). Nevertheless, profile
				// service considers and reports it as "Default", so we add this key in gathered profiles.ini data,
				// to not become this the cause of data mismatch in data comparison later on.
				obj.Default = "1";
			};
			delete obj.iniOrder;
		});
		return true;
	};

	var currentProfDir = _RestMan.dirSrvc.get("ProfD", ci.nsIFile).path;
	var listToUse = null;
	if(profileSrv) {
		var profilesData = new Array(); // Known profile data by current app instance.
		var profilesEnum = profileSrv.profiles;
		while(profilesEnum.hasMoreElements()) {
			var profile = profilesEnum.getNext().QueryInterface(ci.nsIToolkitProfile);
			var profileAttr = {Name: profile.name, Path: profile.rootDir.path};
			try { // ".selectedProfile/.defaultProfile" may crash if there is no profile marked as "default" or is missing.
				var defaultProf = (_RestMan.appInfos.appVer >= 27) ? profileSrv.defaultProfile : profileSrv.selectedProfile;
				if(profile.name == defaultProf.name) {
					profileAttr.Default = "1";
				};
			} catch(ex) {};
			profilesData.push(profileAttr);
		};
		if(readProfilesIni()) {
			if((profilesIniData.length == profilesData.length) &&
			(window.JSON.stringify(profilesIniData) == window.JSON.stringify(profilesData))) { // Profile data comparison.
				listToUse = profilesData;
			} else {
				listToUse = profilesIniData;
				if(mItems) {
					_RestMan.temps.profilesIniData = profilesIniData;
				};
			};
		} else {
			listToUse = profilesData;
		};
	} else if(readProfilesIni()) {
		listToUse = profilesIniData;
		if(mItems) {
			_RestMan.temps.profilesIniData = profilesIniData;
		};
	};

	if(listToUse && (listToUse.length > 0)) {
		var restmanBundle = _RestMan.strings.createBundle("chrome://restartmanager/locale/restartmanager.properties");
		// "restmanBundle_fbk": Fallback for missing strings of incomplete locales, to get them from "en-US".
		var restmanBundle_fbk = _RestMan.strings.createBundle("chrome://restartmanager_fbk/locale/restartmanager.properties");
		listToUse.forEach(function(obj) {
			var menuItemAttr = {};
			if(obj.Default && (obj.Default == "1")) {
				try {
					menuItemAttr.label2 = "( " + restmanBundle.GetStringFromName("default.tag.label") + " )"; // "default".
				} catch(ex) {
					menuItemAttr.label2 = "( " + restmanBundle_fbk.GetStringFromName("default.tag.label") + " )"; // "default".
				};
			};
			var noName = false;
			if(!("Name" in obj) || !obj.Name) {
				noName = true;
			};
			function setProfileState(aState) { // "current", "in_use", "invalid" or "unknown".
				var aStateLocalized = "";
				switch(aState) {
					case "current":
						profilesInUse++;
						try {
							aStateLocalized = restmanBundle.GetStringFromName("current.tag.label"); // "current".
						} catch(ex) {
							aStateLocalized = restmanBundle_fbk.GetStringFromName("current.tag.label"); // "current".
						};
						break;
					case "in_use":
						profilesInUse++;
						try {
							aStateLocalized = restmanBundle.GetStringFromName("inUse.tag.label"); // "in use".
						} catch(ex) {
							aStateLocalized = restmanBundle_fbk.GetStringFromName("inUse.tag.label"); // "in use".
						};
						break;
					case "invalid":
						try {
							aStateLocalized = restmanBundle.GetStringFromName("invalid.tag.label"); // "not valid".
						} catch(ex) {
							aStateLocalized = restmanBundle_fbk.GetStringFromName("invalid.tag.label"); // "not valid".
						};
						break;
					case "unknown":
						try {
							aStateLocalized = restmanBundle.GetStringFromName("unknown.tag.label"); // "unknown state".
						} catch(ex) {
							aStateLocalized = restmanBundle_fbk.GetStringFromName("unknown.tag.label"); // "unknown state".
						};
						break;
				};
				if(menuItemAttr.label2) {
					menuItemAttr.label2 = menuItemAttr.label2.replace(/ \)/, ", " + aStateLocalized + " )");
				} else {
					menuItemAttr.label2 = "( " + aStateLocalized + " )";
				};
				if(noName && (aState !== "invalid")) {
					noName = false;
					return setProfileState("invalid");
				};
				if(aState !== "unknown") {
					menuItemAttr.disabled = true;
				};
			};
			var profState = undefined;
			if((/\.+$/).test(obj.Path)) {
				profState = "invalid";
			} else if(obj.Path == currentProfDir) {
				profState = "current";
				if(("Name" in obj) && obj.Name) {
					currentProfileName = obj.Name;
				};
			} else {
				var profLock = null;
				if((listToUse == profilesData) && profileSrv) {
					if(invalidPaths.indexOf(obj.Path) > -1) {
						profState = "invalid";
					} else try {
						profLock = profileSrv.getProfileByName(obj.Name).lock({});
					} catch(ex) {
						profState = (ex.result == cr.NS_ERROR_FILE_ACCESS_DENIED) ? "in_use" : "invalid";
					};
				} else if(listToUse == profilesIniData) {
					if(!("Path" in obj) || !obj.Path || (invalidPaths.indexOf(obj.Path) > -1)) {
						profState = "invalid";
					} else {
						var dir = Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile", "initWithPath");
						try {
							var profDir = new dir(obj.Path);
						} catch(ex) {
							profState = "invalid";
						};
						if(profDir) {
							if(!profDir.exists() || (invalidPaths.indexOf(obj.Path) > -1)) {
								profState = "invalid";
							} else if(profileSrv && ("lockProfilePath" in profileSrv)) { // Mozilla's "good" guys
								// removed "lockProfilePath"... (Bug 1518575).
								try {
									profLock = profileSrv.lockProfilePath(profDir, profDir);
								} catch(ex) {
									profState = (ex.result == cr.NS_ERROR_FILE_ACCESS_DENIED) ? "in_use" : "invalid";
								};
							} else {
								profState = "unknown";
							};
						};
					};
				};
			};
			if(profLock) {
				profLock.unlock();
			};
			if(("Name" in obj) && obj.Name) {
				menuItemAttr.label = obj.Name;
			} else {
				menuItemAttr.label = "";
			};
			var profDirPath = obj.Path ? obj.Path : "";
			menuItemAttr.ttt = pathStr + ": " + "\"" + profDirPath + "\""; // Tooltip text.
			if(profState) {
				setProfileState(profState);
			} else if(noName) {
				noName = false;
				setProfileState("invalid");
			};
			if(mItems) {
				profMenuList1.appendChild(menuitemConstructor(menuItemAttr));
			};
		});
		if(mItems) {
			if(_RestMan.temps.profilesInUse == 0) {
				_RestMan.temps.profilesInUse += profilesInUse;
			} else {
				_RestMan.temps.profilesInUse = 0;
				_RestMan.temps.profilesInUse = profilesInUse;
			};
			if(profilesInUse > 1) {
				profMenuList1.firstChild.childNodes[1].firstChild.value =
					profMenuList1.firstChild.childNodes[1].firstChild.value.replace(/\d+\//, profilesInUse + "/");
				profMenuList2.firstChild.childNodes[1].firstChild.value =
					profMenuList2.firstChild.childNodes[1].firstChild.value.replace(/\d+\//, profilesInUse + "/");
			};
			if(listToUse.length > 1) {
				profMenuList1.firstChild.childNodes[1].firstChild.value =
					profMenuList1.firstChild.childNodes[1].firstChild.value.replace(/\/\d+/, "/" + listToUse.length);
				profMenuList2.firstChild.childNodes[1].firstChild.value =
					profMenuList2.firstChild.childNodes[1].firstChild.value.replace(/\/\d+/, "/" + listToUse.length);
			};
			for(var n = 0; n < profMenuList1.childNodes.length; n++) {
				if(profMenuList1.childNodes[n].nodeName == "menuitem") {
					profMenuList2.appendChild(profMenuList1.childNodes[n].cloneNode(false));
				};
			};
		} else {
			window.setTimeout(profIniModWatcher, 10000); // "profiles.ini" modification watcher.
			profilesIniData.splice(0, profilesIniData.length);
		};
	};
	return [currentProfileName, profilesInUse];
};

function profileListDestroy(iter, nList) {
	for(null; nList.lastChild.nodeName == "menuitem"; null) {
		nList.removeChild(nList.lastChild);
	};
	nList.firstChild.childNodes[1].firstChild.value = nList.firstChild.childNodes[1].firstChild.value.replace(/\d+/g, 1);
	if(++iter == 2) {
		return;
	};
	nList = profMenuList2;
	_RestMan.temps.profilesIniData = null;
	_RestMan.temps.profilesInUse = 0;
	return profileListDestroy(iter, nList);
};

function profRestartToOrStart(aEvent) {
	var targetProfile = aEvent.target.label;
	var env = cc["@mozilla.org/process/environment;1"].getService(ci.nsIEnvironment);
	function profileLocker() {
		if(!profileSrv) {
			return true;
		};
		var profLock = null;
		try {
			if(_RestMan.temps.profilesIniData) {
				function getPath() {
					for(var o = 0; o < _RestMan.temps.profilesIniData.length; o++) {
						if(_RestMan.temps.profilesIniData[o].Name == targetProfile) {
							return _RestMan.temps.profilesIniData[o].Path;
						};
					};
				};
				var dir = Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile", "initWithPath");
				var profDir = new dir(getPath());
				profLock = profileSrv.lockProfilePath(profDir, profDir);
			} else {
				profLock = profileSrv.getProfileByName(targetProfile).lock({});
			};
		} catch(ex) {
			var profBundle = _RestMan.strings.createBundle("chrome://mozapps/locale/profile/profileSelection.properties");
			var pMissingTitle = "Profile Missing"; // Does not exist in appVer < 4.
			var pLockedTitle = profBundle.GetStringFromName("profileLockedTitle");
			var message = "Your %S profile cannot be loaded. It may be missing or inaccessible."; // Does not exist in appVer < 4.
			try {
				pMissingTitle = profBundle.GetStringFromName("profileMissingTitle"); // appVer 4+.
				message = profBundle.GetStringFromName("profileMissing"); // appVer 4+.
			} catch(ex) {};
			var messageTitle = pMissingTitle + " / " + pLockedTitle;
			message = message.replace(/%\S/, "\"" + targetProfile + "\"");
			_RestMan.promptSrvc.alert(null, messageTitle, message);
			return false;
		};
		profLock.unlock();
		return true;
	};
	function appRunner(restart) {
		var cmdArgs = ["-no-remote", "-P", targetProfile];
		function runAppExec() {
			var nsIProcess = cc["@mozilla.org/process/util;1"].createInstance(ci.nsIProcess);
			var appExec = _RestMan.dirSrvc.get("XREExeF", ci.nsIFile);
			if(appExec.exists()) { // nsIFile.
				nsIProcess.init(appExec);
				if("runw" in nsIProcess) { // appVer 4+.
					return nsIProcess.runw(false, cmdArgs, cmdArgs.length);
				} else {
					return nsIProcess.run(false, cmdArgs, cmdArgs.length);
				};
			};
		};
		if(_RestMan.appInfos.targetOS == "WINNT") {
			if((targetProfile == "ProfileManager") && !restart) {
				cmdArgs.splice(1, 2, "-ProfileManager"); // cmdArgs = ["-no-remote", "-ProfileManager"].
				return runAppExec();
			} else if(profileSrv && ("createInstanceWithProfile" in _RestMan.appStartup) && // appVer 28+.
				!_RestMan.temps.profilesIniData && !restart) {
				for(var cn = 0; cn < targetProfile.length; cn++) {
					if((targetProfile.charCodeAt(cn) > 127) || window.isNaN(targetProfile.charCodeAt(cn))) {
						return runAppExec();
					}; // This issue with non-ascii characters has been resolved in firefox 70 (Bug 1563514).
				};
				return _RestMan.appStartup.createInstanceWithProfile(profileSrv.getProfileByName(targetProfile));
			} else {
				if(restart && !env.get("MOZ_NO_REMOTE")) {
					cmdArgs.shift(); // cmdArgs = ["-P", targetProfile].
				};
				if(targetProfile == "ProfileManager") {
					if(_RestMan.temps.profilesInUse < 2) {
						cmdArgs.splice(0, cmdArgs.length, "-ProfileManager"); // cmdArgs = ["-ProfileManager"].
					} else {
						cmdArgs.splice(1, 2, "-ProfileManager"); // cmdArgs = ["-no-remote", "-ProfileManager"].
					};
				};
				return runAppExec();
			};
		} else {
			cmdArgs.splice(0, 1, "-new-instance"); // cmdArgs = ["-new-instance", "-P", targetProfile].
			if(targetProfile == "ProfileManager") {
				cmdArgs.splice(1, 2, "-ProfileManager"); // cmdArgs = ["-new-instance", "-ProfileManager"].
			};
			return runAppExec();
		};
	};
	if((aEvent.target.parentNode.id == "profMenuList1") && profileLocker()) {
		if(profileSrv && !_RestMan.temps.profilesIniData) {
			var profRootDirPath = profileSrv.getProfileByName(targetProfile).rootDir.path;
			env.set("XRE_PROFILE_PATH", profRootDirPath);
			if(env.get("XRE_PROFILE_PATH") == profRootDirPath) {
				if(_RestMan._RestManOnRestartRequest("restart", "restartOther")) {
					env.set("XRE_PROFILE_NAME", targetProfile);
					env.set("XRE_PROFILE_LOCAL_PATH", profileSrv.getProfileByName(targetProfile).localDir.path);
				} else { // Restart to another profile canceled.
					env.set("XRE_PROFILE_PATH", "");
				};
			} else { // Most likely we are in Windows and there are non-ascii or non-current system language
				// characters in the path, which "nsIEnvironment" can't handle. In such a case, we quit the
				// current profile and start the selected, to make it look like a restart, as normally we do
				// when there is no issue with "nsIEnvironment" to set the environment variables.
				env.set("XRE_PROFILE_PATH", "");
				if(_RestMan._RestManOnRestartRequest("restart", "restartOtherQuit")) {
					return appRunner(true);
				};
			};
		} else if(_RestMan._RestManOnRestartRequest("restart", "restartOtherQuit")) { // If we use "profiles.ini"
			// instead of profiles service, since it's a rather rare case, we use the quit/start method instead of
			// a real restart, to avoid the hassle of finding local dir paths..
			return appRunner(true);
		};
	} else if((aEvent.target.parentNode.id == "profMenuList2") && profileLocker()) {
		return appRunner(false);
	} else if((aEvent.target.id == "rm-rProfman") && _RestMan._RestManOnRestartRequest("restart", "restartOtherQuit")) {
		targetProfile = "ProfileManager";
		return appRunner(true);
	} else if(aEvent.target.id == "rm-sProfman") {
		targetProfile = "ProfileManager";
		return appRunner(false);
	};
};

